import { system, world } from "@minecraft/server";

import { FOOD_SATURATION } from "./food_saturation";

const S_MAX = 20;

system.runInterval(() => {
    const PLAYERS = world.getAllPlayers();

    PLAYERS.forEach((player) => {
        const SATURATION = player.getDynamicProperty("jc:saturation") || 0;
        const HEALTH = player.getComponent("health");
        if (HEALTH.currentValue === 0) return;
        if (HEALTH.currentValue < HEALTH.effectiveMax && SATURATION >= 1) {
            HEALTH.setCurrentValue(HEALTH.currentValue + 1);

            player.setDynamicProperty("jc:saturation", SATURATION - 1);
            player.addEffect("hunger", 3, { amplifier: 255, showParticles: false });
        }
    });
}, 10);

world.afterEvents.itemCompleteUse.subscribe((e) => {
    const { itemStack, source } = e;
    const itemId = itemStack.typeId;

    const saturationValue = FOOD_SATURATION[itemId] || 0;
    if (saturationValue === 0) return;
    const currentSaturation = source.getDynamicProperty("jc:saturation") || 0;

    const newSaturation = Math.min(currentSaturation + saturationValue, S_MAX);

    source.setDynamicProperty("jc:saturation", Number(newSaturation.toPrecision(2)));
});

world.afterEvents.entityDie.subscribe(({ deadEntity }) => {
    if (deadEntity?.typeId !== "minecraft:player") return;
    deadEntity.setDynamicProperty("jc:saturation", 5);
});

world.afterEvents.playerSpawn.subscribe(({ player }) => {
    if (!player.hasTag("jc:saturation_spawned")) {
        player.setDynamicProperty("jc:saturation", 5);
        player.addTag("jc:saturation_spawned");
    }
});

world.beforeEvents.effectAdd.subscribe((e) => {
    if (e.entity?.typeId !== "minecraft:player" || e.effectType !== "Hunger") return;
    if (e.duration > 5) e.cancel = true;
});
